<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\SamsungTV\Packet;

class PacketFactory
{
    /**
     * @link https://gist.github.com/honza889/b70dff4369ff2f0a2afe#file-remote-php-L63
     *
     * @return string  The Start command
     */
    public function getStartPacket(): string
    {
        return pack(
            'C*',
            0x00,
            0x13,
            0x00,
            0x69,
            0x70,
            0x68,
            0x6f,
            0x6e,
            0x65,
            0x2e,
            0x69,
            0x61,
            0x70,
            0x70,
            0x2e,
            0x73,
            0x61,
            0x6d,
            0x73,
            0x75,
            0x6e,
            0x67,
            0x38,
            0x00,
            0x64,
            0x00,
            0x10,
            0x00,
            0x4d,
            0x54,
            0x41,
            0x75,
            0x4d,
            0x43,
            0x34,
            0x77,
            0x4c,
            0x6a,
            0x49,
            0x79,
            0x4d,
            0x67,
            0x3d,
            0x3d,
            0x14,
            0x00,
            0x63,
            0x6d,
            0x46,
            0x75,
            0x5a,
            0x47,
            0x39,
            0x74,
            0x55,
            0x6d,
            0x56,
            0x74,
            0x62,
            0x33,
            0x52,
            0x6c,
            0x53,
            0x55,
            0x51,
            0x3d,
            0x0c,
            0x00,
            0x62,
            0x58,
            0x6c,
            0x53,
            0x5a,
            0x57,
            0x31,
            0x76,
            0x64,
            0x47,
            0x55,
            0x3d
        );
    }

    /**
     * @link https://gist.github.com/honza889/b70dff4369ff2f0a2afe#file-remote-php-L73
     *
     * @param string $keyCode the base64 encoded key code.
     *
     * @return string
     */
    public function getPayload(string $keyCode): string
    {
        return pack("C*", 0x00, 0x00, 0x00, 0x00).pack("C*", strlen($keyCode), 0x00).$keyCode;
    }

    /**
     * @link https://gist.github.com/honza889/b70dff4369ff2f0a2afe#file-remote-php-L75-L76
     *
     * @param string $keyCode the base64 encoded key code.
     *
     * @return string  The final payload to send to the TV.
     */
    public function getKeyPacket(string $keyCode): string
    {
        $payload = $this->getPayload($keyCode);

        return pack(
            "C*",
            0x00,
            0x13,
            0x00,
            0x69,
            0x70,
            0x68,
            0x6f,
            0x6e,
            0x65,
            0x2e,
            0x69,
            0x61,
            0x70,
            0x70,
            0x2e,
            0x73,
            0x61,
            0x6d,
            0x73,
            0x75,
            0x6e,
            0x67
        ) .
            pack("C*", strlen($payload)-1).$payload;
    }
}
